##############################################################################
#
#----------------------------------------------------------------------------------------------------------------------------------------
#   patchcatalog.py
#
#   Classes to read and hold informations for replacements.
#
#   Copyright (c) 2004 Zope Japan KK.(Atsushi Shibata)
#   All Rights Reserved.
#----------------------------------------------------------------------------------------------------------------------------------------
#
# Copyright (c) 2004 Zope Japan KK.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

#Import from replace-dictinaries
from buttons import button_dic
from labels import label_list
from descriptions import desc_list
from replacements import replace_list

#import from local modules
from elementpatcher import elementpatcher,desc,label,button,replace,\
                            tab_domain

import sys
from zLOG import LOG, PROBLEM, WARNING

#import from python modules
from string import find
import re
import os
import fnmatch

__doc__="""patchcatalog.py : Catalog class for ZMI patch'
$Id: patchcatalog.py,v 1.4 2004/12/24 08:13:14 ats_shib Exp $"""

__version__='$Revision: 1.4 $'[11:-2]

kw_filepath =   'filepath'
kw_replacesrc = 'replacesrc'
kw_replacedst = 'replacedst'
kw_descsrc =    'descsrc'
kw_descdst =    'descdst'
kw_labelsrc =   'labelsrc'
kw_labeldst =   'labeldst'
kw_buttonsrc =  'buttonsrc'
kw_buttondst =  'buttondst'

srcs = [kw_replacesrc,kw_descsrc,kw_labelsrc,kw_buttonsrc]
dsts = [kw_replacedst,kw_descdst,kw_labeldst,kw_buttondst]
clss = [replace,desc,label,button]

comment_char = '#'
#Path for 'wild card'
wc_path = '__all__'

#Statuses for reading file
INITIAL = 0
READ_FILE = 1
READ_SRC = 2
READ_DST = 3

elementpat = re.compile('"(.*)"',re.I | re.S)

i18n_dir = 'i18n'
rpo_ext = 'rpo'

class rpoException(Exception):
    """ Exceptions in .rpo file """
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return 'rpo file syntax error: %s' % self.msg

class patchcatalog(dict):
    #Keywords for replace catalog file

    """Class to catalog patch informations and classes"""

    def __init__(self,filename=None):
        self.filename = filename

    def add_patcher(self,idx,path_l,src,dst):
        for path in path_l:
            if not self.has_key(path):
                self[path] = []
            self[path].append(clss[idx](src,dst))

    def read(self):
        """Read replacement informations from file"""
        #File to read
        f = file(self.filename)
        #Initialize status
        st = INITIAL
        #Initialize source kind(index)
        src_idx = -1
        #Initialize file list
        filep = []
        #Initialize source string
        src = ""
        #Initialize line no.
        line_no = 0
        #Go read file...
        for line in f.readlines():
            line = line.lstrip()
            line_no += 1
            if len(line) > 0 and line[0] == '#':
                #skip comment line
                continue
            elif line.find(kw_filepath) == 0:
                #Line is for filepath
                if st != READ_FILE:
                    #Clear list of files
                    filen = eval(line[len(kw_filepath):],globals())
                    filep = [filen]
                else:
                    #Append file to list
                    filen = eval(line[len(kw_filepath):],globals())
                    filep.append(filen)
                st = READ_FILE
            else:
                #Check for source line
                for idx in range(len(srcs)):
                    if line.find(srcs[idx]) == 0:
                        src_idx = idx
                        #Line is for replacesrc
                        if st != READ_SRC:
                            src = line[len(srcs[idx]):]
                            try:
                                src = eval(src,globals())
                            except Exception, msg:
                                raise rpoException(\
                                        '%s (line %d of rpo file %s): \n%s' % \
                                         (msg, line_no, self.filename,src))
                            st = READ_SRC
                            break
                        else:
                            #Format error!!
                            raise rpoException(\
                                    'error in line %d of po file %s' % \
                                    (line_no, self.filename))
                #Check for destination line
                for idx in range(len(dsts)):
                    if line.find(dsts[idx]) == 0:
                        if idx == src_idx:
                            #Line is for replacedst
                            if st != READ_DST:
                                st = READ_DST
                                dst = line[len(srcs[idx]):]
                                try:
                                    dst = eval(dst,globals())
                                except Exception, msg:
                                    raise rpoException(\
                                        '%s (line %d of rpo file %s): \n%s' % \
                                        (msg, line_no, self.filename,dst))
                                self.add_patcher(idx,filep,src,dst)
                                break
                            else:
                                #Format error!!
                                raise rpoException(\
                                     'error in line %d of po file %s' % \
                                     (line_no, self.filename))
                        else:
                            #Format error!!
                            raise rpoException(\
                                 'error in line %d of po file %s' % \
                                 (line_no, self.filename))

    def get_matched(self,path):
        """Return matched patcher object for path"""
        ret_l = []
        if self.has_key(wc_path):
            ret_l += self[wc_path]
        if self.has_key(path):
            ret_l += self[path]
        return ret_l

def build():
    descs = patchcatalog()
    labels = patchcatalog()
    replaces = patchcatalog()
    buttons = patchcatalog()
    
    #Make patcher for descriptions
    for idx in range(0,len(desc_list),2):
        fns = desc_list[idx]
        for fn in fns:
            if not descs.has_key(fn):
                descs[fn] = []
            keys = desc_list[idx+1].keys()
            keys.sort()
            keys.reverse()
            for key in keys:
                #add elementpacher for description
                descs[fn].append(desc(desc_list[idx+1][key],key))
    
    #Make patcher for buttons
    buttons[wc_path] = []
    for key in button_dic.keys():
        buttons[wc_path].append(button(button_dic[key],key))
    
    #Make patcher for from label
    for idx in range(0,len(label_list),2):
        fns = label_list[idx]
        for fn in fns:
            if not labels.has_key(fn):
                labels[fn] = []
            for item in label_list[idx+1]:
                #add elementpacher for label
                labels[fn].append(label(item,item))
    
    #Make patcher for simple replacements
    for idx in range(0,len(replace_list),2):
        fns = replace_list[idx]
        for fn in fns:
            if not replaces.has_key(fn):
                replaces[fn] = []
            for key in replace_list[idx+1]:
                #add elementpacher for description
                replaces[fn].append(replace(key,replace_list[idx+1][key]))

    return descs,labels,replaces,buttons

def read_patch_file(dir):
    pth = os.path.join(dir,i18n_dir)
    if not os.path.isdir(pth):
        return

    #Read the rpo file
    f_names = fnmatch.filter(os.listdir(pth), '*.'+rpo_ext)
    rl = []
    for fn in f_names:
        p = patchcatalog(os.path.join(pth,fn))
        try:
            p.read()
        except:
            exc=sys.exc_info()
            LOG('ZMILocalizer', PROBLEM, 'Replace File has errors',fn, exc)
        rl.append(p)
    return rl


if __name__ == '__main__':
    p = patchcatalog('example.rpo')
    p.read()
    print p
    b = build()
    for p in b:
        print "-" * 40
        print p.get_matched('/lib/python/OFS/dtml/main.dtml')


